<?php
namespace Ondc\Exportcsv\Controller\Getondcproductlist;

use Psr\Log\LoggerInterface;
use Magento\Framework\Stdlib\DateTime\TimezoneInterface;

class Index extends \Magento\Framework\App\Action\Action
{
    const IS_MODULE_ENABLED   = 'ondc/check/active';
    const CSV_EMAIL   = 'ondc/check/csvemail';

    protected $_logLoggerInterface;
    protected $timezone;
    protected $storeManager;
    protected $categoryFactory;
    protected $categoryRepository;
    protected $productModel;
    protected $productFactory;
    protected $productRepository;
    protected $productCollection;
    protected $directoryList;


    /**
     * Constructor
     * @param PageFactory $resultPageFactory
     */
    public function __construct(
        \Magento\Framework\App\Action\Context $context,
        LoggerInterface $logLoggerInterface,
        TimezoneInterface $timezone,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Catalog\Model\CategoryFactory $categoryFactory,
        \Magento\Catalog\Api\CategoryRepositoryInterface $categoryRepository,
        \Magento\Catalog\Model\Product $productModel,
        \Magento\Catalog\Model\ProductFactory $productFactory,
        \Magento\Catalog\Api\ProductRepositoryInterface $productRepository,
        \Magento\Catalog\Model\ResourceModel\Product\CollectionFactory $productCollection,
        \Magento\Framework\App\Filesystem\DirectoryList $directoryList
    )
    {
        $this->_logLoggerInterface = $logLoggerInterface;
        $this->timezone = $timezone;
        $this->categoryFactory = $categoryFactory;
        $this->categoryRepository = $categoryRepository;
        $this->productModel = $productModel;
        $this->storeManager = $storeManager;
        $this->productFactory = $productFactory;
        $this->productRepository = $productRepository;
        $this->productCollection = $productCollection;
        $this->directoryList = $directoryList;
        parent::__construct($context);
    }

    /**
     * Execute action
     * @return ResultInterface
     */
    public function execute()
    {
        
        $store_view_code= 'iit';

        $productWebsiteFilter = 'base,iitspp';

        $categoryNameFilter = 'Grocery';

        $productNameFilter = '%Ondc Demo%';

        $productPriceFilterFrom = 0;

        $productPriceFilterTo = 1000;

        $productCategoryFilter = array(3,4,5); // We can use single category id as well

        $productQtyFilter = 20;

        $isProductCategoryFilter = true;
        $isProductNameFilter = true;
        $isProductPriceFilter = true;
        $isProductQtyFilter = true;

        $productList = $this->productCollection->create();

        $productList->addAttributeToSelect('*')->addAttributeToFilter('attribute_set_id','38')->setFlag('has_stock_status_filter', false);
        if($isProductNameFilter){
            $productList->addAttributeToFilter('name', array('like' => $productNameFilter));
        }
        if($isProductPriceFilter){
            $productList->addFieldToFilter('price', array('from' => $productPriceFilterFrom, 'to' => $productPriceFilterTo) );
        }
        if($isProductCategoryFilter){
            $productList->addCategoriesFilter(['in' => $productCategoryFilter ]);
        }
        if($productQtyFilter){
            $productList->joinField('qty', 'cataloginventory_stock_item', 'qty', 'product_id=entity_id', '{{table}}.stock_id=1', 'left')->addAttributeToFilter('qty',['gteq'=> $productQtyFilter ]);
        }

        $csvHeader = array("sku", "store_view_code", "attribute_set_code", "product_type", "categories", "product_websites", "name", "description", "short_description", "weight", "product_online", "tax_class_name", "visibility", "price", "special_price", "url_key", "meta_title", "meta_keywords", "meta_description", "base_image", "small_image", "thumbnail_image", "gift_message_available", "msrp_display_actual_price_type", "additional_attributes", "qty", "out_of_stock_qty", "use_config_min_qty", "is_qty_decimal", "allow_backorders", "use_config_backorders", "min_cart_qty", "use_config_min_sale_qty", "max_cart_qty", "use_config_max_sale_qty", "is_in_stock", "notify_on_stock_below", "use_config_notify_stock_qty", "manage_stock", "use_config_manage_stock", "use_config_qty_increments", "qty_increments", "use_config_enable_qty_inc", "enable_qty_increments", "is_decimal_divided", "website_id");

        $fileName = 'ondc_product_data_'. $store_view_code .'_'. date('Y-m-d_H-i-s') . '.csv';     
        $filePath =  $this->directoryList->getPath('pub') .'/media/ondc/csv/'. $fileName;
        $file = fopen($filePath, 'w');
        fputcsv($file, $csvHeader,",");

        // Loop through products
        foreach ($productList as $product) {
            try{

                //print_r($product->getData());
                //die;
                
                $attribute_set_code = 'ONDC';
                //$siteUrl = 'http://ondc-buyer-middleware.local/media/catalog/product';
                $siteUrl = 'https://ondcbuyer-magento.stagingshop.com/media/catalog/product';
                $sku = 'csv-'.$product->getSku();
                //$store_view_code = 'default';
                //$attribute_set_code = 'ONDC';
                $product_type = $product->getTypeId();
                $categories = 'Default Category/'.$categoryNameFilter;
                $product_websites = $productWebsiteFilter;
                $name = $product->getName().'-csv';
                $description = $product->getDescription();
                $short_description = $product->getShortDescription();
                $weight = 0.5;
                $product_online = 1;
                $tax_class_name = 'Taxable Goods';
                $visibility = 'Catalog, Search';
                $price = $product->getPrice();
                $special_price = $product->getSpecialPrice();
                $url_key = '';
                $meta_title = $product->getMetaTitle();
                $meta_keywords = $product->getMetaKeywords();
                $meta_description = $product->getMetaDescription();
                if($product->getImage()){
                    $base_image = $siteUrl.$product->getImage();
                }else{
                    $base_image = '';
                }
                if($product->getSmallImage()){
                    $small_image = $siteUrl.$product->getSmallImage();
                }else{
                    $small_image = '';
                }
                if($product->getThumbnail()){
                    $thumbnail_image = $siteUrl.$product->getThumbnail();
                }else{
                    $thumbnail_image = '';
                }                
                $gift_message_available='Use config';
                $msrp_display_actual_price_type='Use config';


                if($product->getOndcAvailableOnCod() == 1){
                    $ondc_available_on_cod='Yes';
                }else{
                    $ondc_available_on_cod='No';
                }

                if($product->getOndcCancellable() == 1){
                    $ondc_cancellable='Yes';
                }else{
                    $ondc_cancellable='No';
                }

                if($product->getOndcReturnable() == 1){
                    $ondc_returnable='Yes';
                }else{
                    $ondc_returnable='No';
                }

                if($product->getOndcProviderTimeLabel() == 1){
                    $ondc_provider_time_label='Yes';
                }else{
                    $ondc_provider_time_label='No';
                }                

                $ondc_seller_name = $product->getOndcSellerName();
                $ondc_domain =$product->getOndcDomain();
                $ondc_bpp_id =$product->getOndcBppId();
                $ondc_bpp_uri =$product->getOndcBppUri();
                $ondc_city =$product->getOndcCity();
                $ondc_common_or_generic_name_of_commodity = $product->getOndcCommonOrGenericNameOfCommodity();
                $ondc_conmpatible_devices = $product->getOndcConmpatibleDevices();
                $ondc_connectivity = $product->getOndcConnectivity();
                $ondc_fulfilment_id = $product->getOndcFulfilmentId();
                $ondc_location_id = $product->getOndcLocationId();
                $ondc_item_id = $product->getOndcItemId();
                $ondc_transaction_id = $product->getOndcTransactionId();
                $ondc_message_id = $product->getOndcMessageId();
                $ondc_seller_logo = $product->getOndcSellerLogo();
                $ondc_seller_short_desc = $product->getOndcSellerShortDesc();
                $ondc_seller_long_desc = $product->getOndcSellerLongDesc();
                $ondc_seller_images = $product->getOndcSellerImages();
                $ondc_provider_id = $product->getOndcProviderId();
                $ondc_provider_location_id = $product->getOndcProviderLocationId();

                
                $additional_attributes = 'ondc_domain='.$ondc_domain;
                $additional_attributes .= ',ondc_available_on_cod='.$ondc_available_on_cod;
                $additional_attributes .= ',ondc_cancellable='.$ondc_cancellable;
                $additional_attributes .= ',ondc_returnable='.$ondc_returnable;
                $additional_attributes .= ',ondc_provider_time_label='.$ondc_provider_time_label;
                $additional_attributes .= ',ondc_bpp_id='.$ondc_bpp_id;
                $additional_attributes .= ',ondc_bpp_uri='.$ondc_bpp_uri;
                $additional_attributes .= ',ondc_city='.$ondc_city;
                $additional_attributes .= ',ondc_common_or_generic_name_of_commodity='.$ondc_common_or_generic_name_of_commodity;
                $additional_attributes .= ',ondc_conmpatible_devices='.$ondc_conmpatible_devices;
                $additional_attributes .= ',ondc_connectivity='.$ondc_connectivity;
                $additional_attributes .= ',ondc_fulfilment_id='.$ondc_fulfilment_id;
                $additional_attributes .= ',ondc_location_id='.$ondc_location_id;
                $additional_attributes .= ',ondc_item_id='.$ondc_item_id;
                $additional_attributes .= ',ondc_transaction_id='.$ondc_transaction_id;
                $additional_attributes .= ',ondc_message_id='.$ondc_message_id;
                $additional_attributes .= ',ondc_seller_name='.$ondc_seller_name;
                $additional_attributes .= ',ondc_seller_logo='.$ondc_seller_logo;
                $additional_attributes .= ',ondc_seller_short_desc='.$ondc_seller_short_desc;
                $additional_attributes .= ',ondc_seller_long_desc='.$ondc_seller_long_desc;
                $additional_attributes .= ',ondc_seller_images='.$ondc_seller_images;
                $additional_attributes .= ',ondc_provider_id='.$ondc_provider_id;
                $additional_attributes .= ',ondc_provider_location_id='.$ondc_provider_location_id;


                //$additional_attributes='color_dropdown=,ondc_available_on_cod=Yes,ondc_bpp_id=ondcqa.demolocations.com,ondc_bpp_uri=https://ondcqa.demolocations.com/ondc/api/,ondc_cancellable=Yes,ondc_city=std:080,ondc_common_or_generic_name_of_commodity=18th,ondc_conmpatible_devices=,ondc_connectivity=,ondc_consume_care_contact_details=Support Team,support@18thdigitech.net,18004254444,ondc_cpu=,ondc_cpu_mfr=,ondc_display=,ondc_domain=nic2004:52110,ondc_form_factor=,ondc_fulfillment_types=Delivery|Self-Pickup|Delivery and Self-Pickup,ondc_fulfilment_id=1,ondc_imported_product_country_of_origin=IND,ondc_item_brand=,ondc_item_code=,ondc_item_color=,ondc_item_id=24-MB03,ondc_item_measure=,ondc_item_symbol=https://ondcqa.demolocations.com/media/catalog/product/placeholder/default/placeholder_4.jpg,ondc_location_id=default_loc,ondc_manufacturer_or_packer_address=Sector 57, Noida,ondc_manufacturer_or_packer_name=18th Digitech,ondc_memory=,ondc_message_id=255fa1e9-db45-4977-a8da-0e93bc967e3b,ondc_model=,ondc_month_year_of_manufacture_packing_import=08/2023,ondc_net_quantity_or_measure_of_commodity_in_pkg=5kg,ondc_non_veg=,ondc_os=,ondc_package_includes=,ondc_parent_item_id=,ondc_product_type=,ondc_provider_address=B-56, Block B, Sector 57, Street 1, Noida, 201301, UP,ondc_provider_id=18th_stores,ondc_provider_location_gps=13.022714275653291,77.64814346152468,ondc_provider_location_id=default_loc,ondc_provider_time_label=No,ondc_refresh=,ondc_returnable=Yes,ondc_return_window=,ondc_screen_size=,ondc_seller_images=https://ondcqa.demolocations.com/media/catalog/product/placeholder/default/placeholder_4.jpg,ondc_seller_logo=https://ondcqa.demolocations.com/media/catalog/product/placeholder/default/placeholder_4.jpg,ondc_seller_long_desc=Online Store,ondc_seller_name=18TH Store,ondc_seller_pickup_return=,ondc_seller_short_desc=Online Store,ondc_special_feature=,ondc_storage=,ondc_style=,ondc_time_to_ship=PT45M,ondc_transaction_id=1c35a962-36da-44f2-9acb-0750497f88e0,ondc_type=,ondc_varient_id=,ondc_varient_name=,ondc_varient_type=,ondc_veg=,size_dropdown=';

                $qty = $product->getQty();
                $out_of_stock_qty=0;
                $use_config_min_qty=1;
                $is_qty_decimal=0;
                $allow_backorders=0;
                $use_config_backorders=1;
                $min_cart_qty=1;
                $use_config_min_sale_qty=1;
                $max_cart_qty = $product->getQty();
                $use_config_max_sale_qty=1;
                $is_in_stock=1;
                $notify_on_stock_below=1;
                $use_config_notify_stock_qty=1;
                $manage_stock=1;
                $use_config_manage_stock=1;
                $use_config_qty_increments=1;
                $qty_increments=1;
                $use_config_enable_qty_inc=1;
                $enable_qty_increments=0;
                $is_decimal_divided=0;
                $website_id=0;

                fputcsv($file, array($sku, $store_view_code, $attribute_set_code, $product_type, $categories, $product_websites, $name, $description, $short_description, $weight, $product_online, $tax_class_name, $visibility, $price, $special_price, $url_key, $meta_title, $meta_keywords, $meta_description, $base_image, $small_image, $thumbnail_image, $gift_message_available, $msrp_display_actual_price_type, $additional_attributes, $qty, $out_of_stock_qty, $use_config_min_qty, $is_qty_decimal, $allow_backorders, $use_config_backorders, $min_cart_qty, $use_config_min_sale_qty, $max_cart_qty, $use_config_max_sale_qty, $is_in_stock, $notify_on_stock_below, $use_config_notify_stock_qty, $manage_stock, $use_config_manage_stock, $use_config_qty_increments, $qty_increments, $use_config_enable_qty_inc, $enable_qty_increments, $is_decimal_divided, $website_id), ",");        
            }
            catch(\Exception $e){
                //echo $sku.'<br>';
                $e->getMessage(); die;
            }
        }
        echo "Product's CSV created successfully for ".$store_view_code." store\n";
        fclose($file );
        
    }
}
